@@ -35,7 +35,6 @@
 # define nonlinear(axis) ((axis)->linked_to_primary != NULL && (axis)->link_udf->at != NULL)
 # define invalid_coordinate(x,y) ((unsigned)(x)==intNaN || (unsigned)(y)==intNaN)
 
-#include <stddef.h>		/* for offsetof() */
 #include "gp_types.h"		/* for TBOOLEAN */
 
 #include "gadgets.h"
@@ -75,13 +74,14 @@ typedef enum AXIS_INDEX {
     V_AXIS,		/* Last index into axis_array[] */
     PARALLEL_AXES,	/* Parallel axis structures are allocated dynamically */
     THETA_index = 1234,	/* Used to identify THETA_AXIS */
+    MOUSE_PROXY_AXIS,	/* Used by watchpoints code */
 
     AXIS_ARRAY_SIZE = PARALLEL_AXES
 } AXIS_INDEX;
 
 
 /* sample axis doesn't need mtics, so use the slot to hold sample interval */
-# define SAMPLE_INTERVAL mtic_freq
+# define SAMPLE_INTERVAL miniticscale
 
 /* What kind of ticmarking is wanted? */
 typedef enum en_ticseries_type {
@@ -121,19 +121,20 @@ typedef struct ticdef {
     TBOOLEAN rangelimited;		/* Limit tics to data range */
     TBOOLEAN enhanced;			/* Use enhanced text mode or labels */
     TBOOLEAN logscaling;		/* place tics using old logscale algorithm */
+    TBOOLEAN force_linear_tics;		/* override the logscaling flag */
 } t_ticdef;
 
-/* we want two auto modes for minitics - default where minitics are
- * auto for log/time and off for linear, and auto where auto for all
- * graphs I've done them in this order so that logscale-mode can
- * simply test bit 0 to see if it must do the minitics automatically.
- * similarly, conventional plot can test bit 1 to see if minitics are
- * required */
+/* Version 5.5 (Jun 2021)
+ * Default is AUTO for log scale, OFF otherwise
+ * New mode TIME only applies when axis or axis tics are set to time mode
+ * and the user has requested "set mxtics time <N> <units>".
+ */
 typedef enum en_minitics_status {
     MINI_OFF,
     MINI_DEFAULT,
     MINI_USER,
-    MINI_AUTO
+    MINI_AUTO,
+    MINI_TIME
 } t_minitics_status;
 
 /* Values to put in the axis_tics[] variables that decides where the
@@ -175,9 +176,9 @@ typedef enum e_constraint {
 /* The unit the tics of a given time/date axis are to interpreted in */
 /* HBB 20040318: start at one, to avoid undershoot */
 typedef enum e_timelevel {
+    TIMELEVEL_DEFAULT = 0,
     TIMELEVEL_SECONDS = 1, TIMELEVEL_MINUTES, TIMELEVEL_HOURS,
-    TIMELEVEL_DAYS, TIMELEVEL_WEEKS, TIMELEVEL_MONTHS,
-    TIMELEVEL_YEARS
+    TIMELEVEL_DAYS, TIMELEVEL_WEEKS, TIMELEVEL_MONTHS, TIMELEVEL_YEARS
 } t_timelevel;
 
 typedef struct axis {
@@ -212,11 +213,11 @@ typedef struct axis {
 
 /* log axis control */
     TBOOLEAN log;		/* log axis stuff: flag "islog?" */
+    int8_t forced_log_link;	/* if "set link" to logscale x/y: x1/y1: -1  x2/y2: 1 */
     double base;		/* logarithm base value */
     double log_base;		/* ln(base), for easier computations */
 
-/* linked axis information (used only by x2, y2)
- * If axes are linked, the primary axis info will be cloned into the
+/* If axes are linked, the primary axis info will be cloned into the
  * secondary axis only up to this point in the structure.
  */
     struct axis *linked_to_primary;	/* Set only in a secondary axis */
@@ -225,13 +226,14 @@ typedef struct axis {
 
 /* ticmark control variables */
     int ticmode;		/* tics on border/axis? mirrored? */
-    struct ticdef ticdef;	/* tic series definition */
-    int tic_rotate;		/* ticmarks rotated by this angle */
     enum JUSTIFY tic_pos;	/* left/center/right tic label justification */
+    struct ticdef ticdef;	/* tic series definition */
+    float tic_rotate;		/* ticmarks rotated by this angle */
     TBOOLEAN gridmajor;		/* Grid lines wanted on major tics? */
     TBOOLEAN gridminor;		/* Grid lines for minor tics? */
-    t_minitics_status minitics;	/* minor tic mode (none/auto/user)? */
-    double mtic_freq;		/* minitic stepsize */
+    t_minitics_status minitics;	/* minor tic mode (none/auto/user/time) */
+    t_timelevel minitic_units;	/* in time mode, DAYS/MONTHS/etc */
+    int mtic_freq;		/* number of minitics per interval */
     double ticscale;		/* scale factor for tic marks (was (0..1])*/
     double miniticscale;	/* and for minitics */
     double ticstep;		/* increment used to generate tic placement */
@@ -240,6 +242,7 @@ typedef struct axis {
 /* time/date axis control */
     td_type datatype;		/* {DT_NORMAL|DT_TIMEDATE} controls _input_ */
     td_type tictype;		/* {DT_NORMAL|DT_TIMEDATE|DT_DMS} controls _output_ */
+    t_timelevel tic_units;	/* granularity of auto-generated time tics */
     char *formatstring;		/* the format string for output */
     char *ticfmt;		/* autogenerated alternative to formatstring (needed??) */
     t_timelevel timelevel;	/* minimum time unit used to quantize ticks */
@@ -254,7 +257,7 @@ typedef struct axis {
     double paxis_x;		/* x coordinate of parallel axis */
 } AXIS;
 
-#define DEFAULT_AXIS_TICDEF {TIC_COMPUTED, NULL, {TC_DEFAULT, 0, 0.0}, {NULL, {0.,0.,0.}, FALSE},  { character, character, character, 0., 0., 0. }, FALSE, TRUE, FALSE }
+#define DEFAULT_AXIS_TICDEF {TIC_COMPUTED, NULL, {TC_DEFAULT, 0, 0.0}, {NULL, {0.,0.,0.}, FALSE},  { character, character, character, 0., 0., 0. }, FALSE, TRUE, FALSE, FALSE }
 #define DEFAULT_AXIS_ZEROAXIS {0, LT_AXIS, 0, DASHTYPE_AXIS, 0, 0, 1.0, PTSZ_DEFAULT, DEFAULT_P_CHAR, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}
 
 #define DEFAULT_AXIS_STRUCT {						    \
@@ -269,16 +272,19 @@ typedef struct axis {
 	0, 0,   		/* terminal lower and upper coords */	    \
 	0.,        		/* terminal scale */			    \
 	0,        		/* zero axis position */		    \
-	FALSE, 10.0, 0.0,	/* log, base, log(base) */		    \
+	FALSE, 0,		/* log, forced_log_link */		    \
+	10.0, 0.0,		/* base, log(base) */			    \
 	NULL, NULL,		/* linked_to_primary, linked_to_secondary */\
 	NULL,      		/* link function */                         \
 	NO_TICS,		/* tic output positions (border, mirror) */ \
+	CENTRE,		 	/* horizontal justification */		    \
 	DEFAULT_AXIS_TICDEF,	/* tic series definition */		    \
-	0, CENTRE,	 	/* tic_rotate, horizontal justification */  \
+	0.0,			/* tic_rotate */			    \
 	FALSE, FALSE,	 	/* grid{major,minor} */			    \
-	MINI_DEFAULT, 10.,	/* minitics, mtic_freq */		    \
+	MINI_DEFAULT, 1, 10,	/* minitics, units, mtic_freq */	    \
 	1.0, 0.5, 0.0, TRUE,	/* ticscale, miniticscale, ticstep, tic_in */ \
 	DT_NORMAL, DT_NORMAL,	/* datatype for input, output */	    \
+	TIMELEVEL_SECONDS,	/* default granularity (seconds) for tics */\
 	NULL, NULL,      	/* output format, another output format */  \
 	0,			/* timelevel */                             \
 	0,			/* index (e.g.FIRST_Y_AXIS) */		    \
@@ -320,6 +326,7 @@ extern const struct ticdef default_axis_ticdef;
 /* default format for tic mark labels */
 #define DEF_FORMAT "% h"
 #define DEF_FORMAT_LATEX "$%h$"
+#define is_def_format(s) (!strcmp(s,"% h") || !strcmp(s,"% H"))
 
 /* default parse timedata string */
 #define TIMEFMT "%d/%m/%y,%H:%M"
@@ -360,16 +367,13 @@ extern int tic_text, rotate_tics, tic_hjust, tic_vjust;
 extern struct lp_style_type grid_lp; /* linestyle for major grid lines */
 extern struct lp_style_type mgrid_lp; /* linestyle for minor grid lines */
 
-extern double polar_grid_angle; /* angle step in polar grid in radians */
+extern double theta_grid_angle; /* angle step in polar grid in radians */
 extern double theta_origin;	/* 0 = right side of plot */
 extern double theta_direction;	/* 1 = counterclockwise -1 = clockwise */
 
-/* Length of the longest tics label, set by widest_tic_callback(): */
+/* Length of the longest tics label, set by widest_tic_callback() */
 extern int widest_tic_strlen;
 
-/* flag to indicate that in-line axis ranges should be ignored */
-extern TBOOLEAN inside_zoom;
-
 /* axes being used by the current plot */
 extern AXIS_INDEX x_axis, y_axis, z_axis;
 
@@ -478,15 +482,21 @@ TBOOLEAN bad_axis_range(struct axis *axis);
 
 void save_writeback_all_axes(void);
 int  parse_range(AXIS_INDEX axis);
-void parse_skip_range(void);
+TBOOLEAN parse_skip_range(void);
 void check_axis_reversed(AXIS_INDEX axis);
+double parse_one_range_limit( double default_value );
 
 /* set widest_tic_label: length of the longest tics label */
 void widest_tic_callback(struct axis *, double place, char *text, int ticlevel,
 			struct lp_style_type grid, struct ticmark *);
+/* callback that just counts the number of tics that are placed */
+void tic_count_callback(struct axis *, double place, char *text, int ticlevel,
+			struct lp_style_type grid, struct ticmark *);
+void sanity_check_log_tics( int axis_index );
 
 void get_position(struct position *pos);
-void get_position_default(struct position *pos, enum position_type default_type, int ndim);
+void get_position_default(struct position *pos, enum position_type default_type,
+			TBOOLEAN clear, int ndim);
 
 void gstrdms(char *label, char *format, double value);
 
@@ -496,9 +506,11 @@ void extend_primary_ticrange(AXIS *axis);
 void update_primary_axis_range(struct axis *secondary);
 void update_secondary_axis_range(struct axis *primary);
 void reconcile_linked_axes(AXIS *primary, AXIS *secondary);
+void extend_autoscaled_log_axis(AXIS *primary);
 
 int map_x(double value);
 int map_y(double value);
+int map_ysharp(double value);
 
 double map_x_double(double value);
 double map_y_double(double value);
