@@ -48,6 +48,7 @@
 #include "version.h"
 #include "voxelgrid.h"
 #include "encoding.h"
+#include "save.h"
 #include "xdg.h"
 
 #include <signal.h>
@@ -82,9 +83,7 @@
 */
 #ifdef HAVE_LIBEDITLINE
 #  include <editline/readline.h>
-#  ifdef GNUPLOT_HISTORY
-#    include <histedit.h>
-#  endif
+#  include <histedit.h>
 #endif
 
 /* enable gnuplot history with readline */
@@ -295,38 +294,39 @@ main(int argc_orig, char **argv)
     }
 #endif /* MSDOS || OS2 */
 
-#if (defined(PIPE_IPC) || defined(_WIN32)) && (defined(HAVE_LIBREADLINE) || (defined(HAVE_LIBEDITLINE) && defined(X11)))
+#if (defined(PIPE_IPC) || defined(_WIN32)) && (defined(HAVE_LIBREADLINE) || (defined(HAVE_LIBEDITLINE)))
     /* Editline needs this to be set before the very first call to readline(). */
-    /* Support for rl_getc_function is broken for utf-8 in editline. Since it is only
-       really required for X11, disable this section when building without X11. */
     rl_getc_function = getc_wrapper;
 #endif
 
-#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
+#if defined(HAVE_LIBREADLINE)
     /* T.Walter 1999-06-24: 'rl_readline_name' must be this fix name.
      * It is used to parse a 'gnuplot' specific section in '~/.inputrc'
      * or gnuplot specific commands in '.editrc' (when using editline
      * instead of readline) */
     rl_readline_name = "Gnuplot";
     rl_terminal_name = getenv("TERM");
-#if defined(HAVE_LIBREADLINE)
     using_history();
-#else
-    history_init();
-#endif
-#endif
 
-#if defined(HAVE_LIBREADLINE) && defined(RL_VERSION_MAJOR)
+    #if !defined(MISSING_RL_TILDE_EXPANSION)
+    rl_complete_with_tilde_expansion = 1;
+    #endif
+
+    #if defined(RL_VERSION_MAJOR)
     /* Starting with readline v8.1 bracketed paste mode is on by default.
      * This breaks multi-line pasted input to gnuplot because it looks like
      * one long run-on line.
      */
     if (RL_VERSION_MAJOR >= 8)
 	rl_variable_bind ("enable-bracketed-paste", "off");
+    #endif
 #endif
 
-#if defined(HAVE_LIBREADLINE) && !defined(MISSING_RL_TILDE_EXPANSION)
-    rl_complete_with_tilde_expansion = 1;
+#if defined(HAVE_LIBEDITLINE)
+    rl_readline_name = "Gnuplot";
+    rl_terminal_name = getenv("TERM");
+    el = el_init("gnuplot", stdin, stdout, stderr);
+    history_init();
 #endif
 
     for (i = 1; i < argc; i++) {
@@ -407,8 +407,10 @@ main(int argc_orig, char **argv)
     /* "pi" is hard-wired as the first variable */
     (void) add_udv_by_name("GNUTERM");
     (void) add_udv_by_name("I");
+    (void) add_udv_by_name("Inf");
     (void) add_udv_by_name("NaN");
     init_constants();
+    /* user-defined variables start immediately after NaN */
     udv_user_head = &(udv_NaN->next_udv);
 
     init_memory();
@@ -748,6 +750,8 @@ init_constants()
     (void) Gcomplex(&(udv_NaN->udv_value), not_a_number(), 0.0);
     udv_I = get_udv_by_name("I");
     (void) Gcomplex(&(udv_I->udv_value), 0.0, 1.0);
+    udv_Inf = get_udv_by_name("Inf");
+    (void) Gcomplex(&(udv_Inf->udv_value), INFINITY, 0.0);
 }
 
 /*
@@ -788,6 +792,20 @@ init_session()
 
 	load_rcfile(2);		/* ~/.gnuplot */
 	load_rcfile(3);		/* ~/.config/gnuplot/gnuplotrc */
+
+#if !defined(WIN32) && !defined(OS2) && !defined(MSDOS)
+	/* Save initial state variables to a file so that later
+	 * "save changes" can use them to determine what has changed.
+	 */
+	if (savefp == NULL) {
+	    if (!(savefp = tmpfile()))
+		perror("Cannot open initial state save file");
+	    else {
+		save_all(savefp);
+		fflush(savefp);
+	    }
+	}
+#endif
 }
 
 /*
