@@ -39,18 +39,23 @@
 #include "setshow.h"
 
 #include "alloc.h"
-#include "axis.h"
 #include "command.h"
 #include "contour.h"
+#include "datablock.h"
 #include "datafile.h"
+#include "encoding.h"
 #include "eval.h"
+#include "filters.h"
 #include "fit.h"
 #include "gp_time.h"
 #include "graphics.h"
+#include "help.h"
 #include "hidden3d.h"
 #include "jitter.h"
 #include "misc.h"
 #include "gp_hist.h"
+#include "gplocale.h"
+#include "loadpath.h"
 #include "plot2d.h"
 #include "plot3d.h"
 #include "save.h"
@@ -58,9 +63,9 @@
 #include "tabulate.h"
 #include "util.h"
 #include "term_api.h"
-#include "variable.h"
 #include "version.h"
 #include "voxelgrid.h"
+#include "watch.h"
 #ifdef USE_MOUSE
 # include "mouse.h"
 #endif
@@ -99,11 +104,11 @@ static void show_style_rectangle(void);
 static void show_style_circle(void);
 static void show_style_ellipse(void);
 static void show_grid(void);
+static void show_help(void);
 static void show_raxis(void);
 static void show_paxis(void);
 static void show_zeroaxis(AXIS_INDEX);
 static void show_label(int tag);
-static void show_keytitle(void);
 static void show_key(void);
 static void show_logscale(void);
 static void show_offsets(void);
@@ -114,7 +119,6 @@ static void show_parametric(void);
 static void show_pm3d(void);
 static void show_palette(void);
 static void show_palette_rgbformulae(void);
-static void show_palette_fit2rgbformulae(void);
 static void show_palette_palette(void);
 static void show_palette_gradient(void);
 static void show_palette_colornames(void);
@@ -131,6 +135,7 @@ static void show_psdir(void);
 static void show_angles(void);
 static void show_samples(void);
 static void show_isosamples(void);
+static void show_isotropic(void);
 static void show_view(void);
 static void show_surface(void);
 static void show_hidden3d(void);
@@ -148,6 +153,7 @@ static void show_range(AXIS_INDEX axis);
 static void show_link(void);
 static void show_nonlinear(void);
 static void show_xyzlabel(const char *name, const char *suffix, text_label * label);
+static void show_text_justification( JUSTIFY just );
 static void show_title(void);
 static void show_axislabel(AXIS_INDEX);
 static void show_data_is_timedate(AXIS_INDEX);
@@ -158,6 +164,7 @@ static void show_fontpath(void);
 static void show_zero(void);
 static void show_datafile(void);
 static void show_table(void);
+static void show_imaginary_i(void);
 static void show_micro(void);
 static void show_minus_sign(void);
 static void show_mouse(void);
@@ -165,7 +172,7 @@ static void show_plot(void);
 static void show_variables(void);
 
 static void show_linestyle(int tag);
-static void show_linetype(struct linestyle_def *listhead, int tag);
+static void show_linetype(struct linestyle_def *listhead);
 static void show_arrowstyle(int tag);
 static void show_arrow(int tag);
 
@@ -173,6 +180,7 @@ static void show_ticdef(AXIS_INDEX);
 static void show_ticdefp(struct axis *);
        void show_position(struct position * pos, int ndim);
 static void show_functions(void);
+static void show_warnings(void);
 
 static int var_show_all = 0;
 
@@ -235,6 +243,9 @@ show_command()
     case S_CNTRLABEL:
 	show_contour();
 	break;
+    case S_CONTOURFILL:
+	save_contourfill(stderr);
+	break;
     case S_DEBUG:
 	fprintf(stderr,"debug level is %d\n",debug);
 	break;
@@ -259,6 +270,9 @@ show_command()
     case S_GRID:
 	show_grid();
 	break;
+    case S_HELP:
+	show_help();
+	break;
     case S_RAXIS:
 	show_raxis();
 	break;
@@ -311,16 +325,13 @@ show_command()
 	show_linestyle(tag);
 	break;
     case S_LINETYPE:
-	CHECK_TAG_GT_ZERO;
-	show_linetype(first_perm_linestyle, tag);
+	show_linetype(first_perm_linestyle);
 	break;
     case S_MONOCHROME:
 	fprintf(stderr,"monochrome mode is %s\n", monochrome ? "active" : "not active");
-	if (equals(c_token,"lt") || almost_equals(c_token,"linet$ype")) {
+	if (equals(c_token,"lt") || almost_equals(c_token,"linet$ype"))
 	    c_token++;
-	    CHECK_TAG_GT_ZERO;
-	}
-	show_linetype(first_mono_linestyle, tag);
+	show_linetype(first_mono_linestyle);
 	break;
     case S_DASHTYPE:
 	CHECK_TAG_GT_ZERO;
@@ -335,6 +346,9 @@ show_command()
     case S_KEY:
 	show_key();
 	break;
+    case S_I_SYMBOL:
+	show_imaginary_i();
+	break;
     case S_LOGSCALE:
 	show_logscale();
 	break;
@@ -374,6 +388,10 @@ show_command()
     case S_COLORBOX:
 	show_colorbox();
 	break;
+    case S_COLORMAP:
+	save_colormaps(stderr);
+	c_token++;
+	break;
     case S_COLORNAMES:
     case S_COLORSEQUENCE:
 	c_token--;
@@ -418,6 +436,9 @@ show_command()
     case S_WALL:
 	save_walls(stderr);
 	break;
+    case S_WARNINGS:
+	show_warnings();
+	break;
     case S_ANGLES:
 	show_angles();
 	break;
@@ -430,6 +451,9 @@ show_command()
     case S_ISOSAMPLES:
 	show_isosamples();
 	break;
+    case S_ISOTROPIC:
+	show_isotropic();
+	break;
     case S_ISOSURFACE:
 	show_isosurface();
 	break;
@@ -612,7 +636,6 @@ show_command()
     case S_VARIABLES:
 	show_variables();
 	break;
-/* FIXME: get rid of S_*DTICS, S_*MTICS cases */
     case S_XTICS:
     case S_XDTICS:
     case S_XMTICS:
@@ -651,7 +674,7 @@ show_command()
 	break;
 
     case S_MULTIPLOT:
-	fprintf(stderr,"multiplot mode is %s\n", multiplot ? "on" : "off");
+	fprintf(stderr,"multiplot mode is %s\n", (in_multiplot > 0) ? "on" : "off");
 	break;
 
     case S_TERMOPTIONS:
@@ -660,21 +683,27 @@ show_command()
 	break;
 
     case S_THETA:
-	fprintf(stderr,"Theta increases %s with origin at %s of plot\n",
+	fprintf(stderr,"\tTheta increases %s with origin at %s of plot\n",
 		theta_direction > 0 ? "counterclockwise" : "clockwise",
 		theta_origin == 180 ? "left" : theta_origin ==  90 ? "top" :
 		theta_origin == -90 ? "bottom" : "right");
 	break;
 
-    /* HBB 20010525: 'set commands' that don't have an
-     * accompanying 'show' version, for no particular reason: */
-    /* --- such case now, all implemented. */
+    case S_WATCH:
+	show_watchpoints();
+	break;
 
     case S_INVALID:
-	error_message = "Unrecognized option. See 'help show'.";
-	break;
     default:
-	error_message = "invalid or deprecated syntax";
+#ifdef WITH_CHI_SHAPES
+	if (almost_equals(c_token,"chi$_shapes")) {
+	    fprintf(stderr,"Default χ-shape parameter = %g of longest edge in convex hull\n",
+		chi_shape_default_fraction);
+	    c_token++;
+	    break;
+	}
+#endif
+	error_message = "Unrecognized option.";
 	break;
     }
 
@@ -696,7 +725,6 @@ show_at()
 {
     (void) putc('\n', stderr);
     disp_at(temp_at(), 0);
-    c_token++;
 }
 
 
@@ -768,6 +796,9 @@ disp_at(struct at_type *curr_at, int level)
 	case DOLLARS:
 	    fprintf(stderr, " %d\n", (int)(arg->v_arg.v.int_val));
 	    break;
+	case EVAL:
+	    fprintf(stderr, " function block %s\n", arg->udv_arg->udv_name);
+	    break;
 	default:
 	    (void) putc('\n', stderr);
 	}
@@ -941,6 +972,29 @@ show_version(FILE *fp)
 		"";
 #endif
 
+	    const char *libamos =
+#ifdef HAVE_AMOS
+		"+AMOS  ";
+#else
+		"";
+#endif
+
+	    const char *have_cexint =
+#if defined(HAVE_ZEXINT)
+		"+ZEXINT  ";
+#elif defined(HAVE_CEXINT)
+		"+CEXINT  ";
+#else
+		"";
+#endif
+
+	    const char *complexfunc =
+#ifdef HAVE_COMPLEX_FUNCS
+		"+COMPLEX_FUNCS  ";
+#else
+		"";
+#endif
+
 	    const char *libgd =
 #ifdef HAVE_LIBGD
 # ifdef HAVE_GD_PNG
@@ -963,19 +1017,8 @@ show_version(FILE *fp)
 #endif
 		"";
 
-	    const char *linuxvga =
-#ifdef LINUXVGA
-		"+LINUXVGA  "
-#endif
-		"";
-
 	    const char *nocwdrc =
-#ifdef USE_CWDRC
-		"+"
-#else
-		"-"
-#endif
-		"USE_CWDRC  ";
+		"-USE_CWDRC  ";
 
 	    const char *x11 =
 #ifdef X11
@@ -999,15 +1042,39 @@ show_version(FILE *fp)
 	    const char *plotoptions=
 		"+OBJECTS  "
 #ifdef USE_STATS
-		"+STATS "
+		"+STATS  "
 #else
-		"-STATS "
+		"-STATS  "
+#endif
+#ifdef USE_WATCHPOINTS
+		"+WATCHPOINTS  "
+#else
+		"-WATCHPOINTS  "
 #endif
 #ifdef HAVE_EXTERNAL_FUNCTIONS
 		"+EXTERNAL_FUNCTIONS "
+#endif
+#ifdef USE_POLAR_GRID
+		"+POLARGRID "
+#else
+		"-POLARGRID "
 #endif
 	    "";
 
+	    const char *fblocks =
+#if defined(USE_FUNCTIONBLOCKS)
+		"+FUNCTIONBLOCKS ";
+#else
+		"-FUNCTIONBLOCKS ";
+#endif
+
+	    const char *chi_shapes =
+#if defined(WITH_CHI_SHAPES)
+		"+CHI_SHAPES ";
+#else
+		"-CHI_SHAPES ";
+#endif
+
 	    const char *unicodebuild =
 #if defined(_WIN32) && defined(UNICODE)
 		"+UNICODE  ";
@@ -1015,10 +1082,13 @@ show_version(FILE *fp)
 		"";
 #endif
 
-	    sprintf(compile_options, "    %s%s\n    %s%s\n    %s%s%s\n    %s%s%s%s\n",
+	    sprintf(compile_options,
+		    "    %s%s\n    %s%s\n    %s%s%s%s\n    %s\n    %s%s%s%s\n    %s%s\n",
 		    rdline, gnu_rdline, unicodebuild, plotoptions,
-		    libcerf, libgd, linuxvga,
-		    nocwdrc, x11, use_mouse, hiddenline
+		    complexfunc, libcerf, libamos, have_cexint,
+		    libgd,
+		    nocwdrc, x11, use_mouse, hiddenline,
+		    fblocks, chi_shapes
 		    );
 	}
 
@@ -1034,11 +1104,7 @@ show_version(FILE *fp)
 	p += sizeof(prefix) - 1;
     } else {
 #ifdef BINDIR
-# ifdef X11
-	fprintf(fp, "#!%s/gnuplot -persist\n#\n", BINDIR);
-#  else
 	fprintf(fp, "#!%s/gnuplot\n#\n", BINDIR);
-# endif				/* not X11 */
 #endif /* BINDIR */
     }
 
@@ -1080,19 +1146,33 @@ show_version(FILE *fp)
     /* show version long */
     if (almost_equals(c_token, "l$ong")) {
 
+	/* We will use this to test for packing holes in struct coordinate */
+	struct gen_coord { coordval dummy[7]; enum coord_type type; };
+
 	c_token++;
 	fprintf(stderr, "\nCompile options:\n%s", compile_options);
-	fprintf(stderr, "    %d-bit integer arithmetic\n\n",(int)sizeof(intgr_t)*8);
+	fprintf(stderr, "    %d-bit integer arithmetic\n",(int)sizeof(intgr_t)*8);
+
+#ifdef WITH_EXTRA_COORDINATE
+	fprintf(stderr, "    sizeof(struct coordinate) = %d with extra coordinate\n",
+		(int)(sizeof(struct coordinate)));
+	if (sizeof(struct coordinate) > sizeof(struct gen_coord))
+	    fprintf(stderr, "\tThis is a waste of space. "\
+	    "You should reconfigure using --without-extra-coordinate\n");
+#else
+	fprintf(stderr, "    sizeof(struct coordinate) = %d without extra coordinate\n",
+		(int)(sizeof(struct coordinate)));
+#endif
 
-#ifdef X11
 	{
 	    char *driverdir = getenv("GNUPLOT_DRIVER_DIR");
-
-	    if (driverdir == NULL)
-		driverdir = X11_DRIVER_DIR;
-	    fprintf(stderr, "GNUPLOT_DRIVER_DIR = \"%s\"\n", driverdir);
-	}
+#ifdef GNUPLOT_DRIVER_DIR
+	    if (!driverdir)
+		driverdir = GNUPLOT_DRIVER_DIR;
 #endif
+	    fprintf(stderr, "GNUPLOT_DRIVER_DIR = \"%s\"\n",
+		    driverdir ? driverdir : "");
+	}
 
 	{
 	    char *psdir = getenv("GNUPLOT_PS_DIR");
@@ -1302,7 +1382,7 @@ show_contour()
 	    (draw_contour) ? "drawn" : "not drawn\n");
 
     if (draw_contour) {
-	fprintf(stderr, " in %d levels on ", contour_levels);
+	fprintf(stderr, " in %d levels on ", contour_params.levels);
 	switch (draw_contour) {
 	case CONTOUR_BASE:
 	    fputs("grid base\n", stderr);
@@ -1317,35 +1397,35 @@ show_contour()
 	    /* should not happen --- be easy: don't complain... */
 	    break;
 	}
-	switch (contour_kind) {
+	switch (contour_params.kind) {
 	case CONTOUR_KIND_LINEAR:
 	    fputs("\t\tas linear segments\n", stderr);
 	    break;
 	case CONTOUR_KIND_CUBIC_SPL:
-	    fprintf(stderr, "\t\tas cubic spline interpolation segments with %d pts\n", contour_pts);
+	    fprintf(stderr, "\t\tas cubic spline interpolation segments with %d pts\n", contour_params.npoints);
 	    break;
 	case CONTOUR_KIND_BSPLINE:
-	    fprintf(stderr, "\t\tas bspline approximation segments of order %d with %d pts\n", contour_order, contour_pts);
+	    fprintf(stderr, "\t\tas bspline approximation segments of order %d with %d pts\n", contour_params.order, contour_params.npoints);
 	    break;
 	}
-	switch (contour_levels_kind) {
+	switch (contour_params.levels_kind) {
 	case LEVELS_AUTO:
-	    fprintf(stderr, "\t\tapprox. %d automatic levels\n", contour_levels);
+	    fprintf(stderr, "\t\tapprox. %d automatic levels\n", contour_params.levels);
 	    break;
 	case LEVELS_DISCRETE:
 	    {
 		int i;
-		fprintf(stderr, "\t\t%d discrete levels at ", contour_levels);
+		fprintf(stderr, "\t\t%d discrete levels at ", contour_params.levels);
 		fprintf(stderr, "%g", contour_levels_list[0]);
-		for (i = 1; i < contour_levels; i++)
+		for (i = 1; i < contour_params.levels; i++)
 		    fprintf(stderr, ",%g ", contour_levels_list[i]);
 		putc('\n', stderr);
 		break;
 	    }
 	case LEVELS_INCREMENTAL:
-	    fprintf(stderr, "\t\t%d incremental levels starting at %g, step %g, end %g\n", contour_levels, contour_levels_list[0],
+	    fprintf(stderr, "\t\t%d incremental levels starting at %g, step %g, end %g\n", contour_params.levels, contour_levels_list[0],
 		    contour_levels_list[1],
-		    contour_levels_list[0] + (contour_levels - 1) * contour_levels_list[1]);
+		    contour_levels_list[0] + (contour_params.levels - 1) * contour_levels_list[1]);
 	    /* contour-levels counts both ends */
 	    break;
 	}
@@ -1354,14 +1434,14 @@ show_contour()
 	fprintf(stderr, "\tcontour lines are drawn in %s linetypes\n",
 		clabel_onecolor ? "the same" : "individual");
 	fprintf(stderr, "\tformat for contour labels is '%s' font '%s'\n",
-		contour_format, clabel_font ? clabel_font : "");
+		contour_params.format, clabel_font ? clabel_font : "");
 	fprintf(stderr, "\ton-plot labels placed at segment %d with interval %d\n",
 		clabel_start, clabel_interval);
-	if (contour_firstlinetype > 0)
-		fprintf(stderr, "\tfirst contour linetype will be %d\n", contour_firstlinetype);
+	if (contour_params.firstlinetype > 0)
+		fprintf(stderr, "\tfirst contour linetype will be %d\n", contour_params.firstlinetype);
 	else
 		fprintf(stderr, "\tfirst contour linetype will be chosen automatically\n");
-	fprintf(stderr, "\tcontour levels will be %ssorted\n", contour_sortlevels ? "" : "un");
+	fprintf(stderr, "\tcontour levels will be %ssorted\n", contour_params.sortlevels ? "" : "un");
     }
 }
 
@@ -1413,7 +1493,7 @@ show_dgrid3d()
 		reverse_table_lookup(dgrid3d_mode_tbl, dgrid3d_mode),
 		dgrid3d_x_scale,
 		dgrid3d_y_scale,
-		dgrid3d_kdensity ? ", kdensity2d mode" : "" );
+		dgrid3d_kdensity ? ", kdensity mode" : "" );
       }
     else
 	fputs("\tdata grid3d is disabled\n", stderr);
@@ -1551,15 +1631,19 @@ show_style()
 	show_style_ellipse();
 	c_token++;
 	break;
+    case SHOW_STYLE_WATCHPOINT:
+	show_style_watchpoint();
+	c_token++;
+	break;
     default:
 	/* show all styles */
 	show_styles("Data",data_style);
 	show_styles("Functions", func_style);
 	show_linestyle(0);
 	show_fillstyle();
-	show_increment();
 	show_histogram();
 	show_textbox();
+	show_style_watchpoint();
 	save_style_parallel(stderr);
 	show_arrowstyle(0);
 	show_boxplot();
@@ -1656,7 +1740,7 @@ show_grid()
 
     /* HBB 20010806: new storage method for grid options: */
     fprintf(stderr, "\t%s grid drawn at",
-	    (polar_grid_angle != 0) ? "Polar" : "Rectangular");
+	    (theta_grid_angle != 0) ? "Polar" : "Rectangular");
 #define SHOW_GRID(axis)						\
     if (axis_array[axis].gridmajor)				\
 	fprintf(stderr, " %s", axis_name(axis));	\
@@ -1680,9 +1764,9 @@ show_grid()
     fputc('\n', stderr);
     if (grid_vertical_lines)
 	fprintf(stderr, "\tVertical grid lines in 3D plots\n");
-    if (polar_grid_angle)
+    if (theta_grid_angle)
 	fprintf(stderr, "\tGrid radii drawn every %f %s\n",
-		polar_grid_angle / ang2rad,
+		theta_grid_angle / ang2rad,
 		(ang2rad == 1.0) ? "radians" : "degrees");
     if (grid_spiderweb)
 	fprintf(stderr, "\tGrid shown in spiderplots\n");
@@ -1690,6 +1774,16 @@ show_grid()
     fprintf(stderr, "\tGrid drawn at %s\n", (grid_layer==-1) ? "default layer" : ((grid_layer==0) ? "back" : "front"));
 }
 
+/* process 'show help' command */
+static void
+show_help()
+{
+#ifndef NO_GIH
+    fprintf(stderr,"\thelp subtopics are sorted by %s\n",
+	help_sort_by_rows ? "row" : "column");
+#endif
+}
+
 static void
 show_raxis()
 {
@@ -1748,49 +1842,7 @@ show_label(int tag)
 		    this_label->tag,
 		    (this_label->text==NULL) ? "" : conv_text(this_label->text));
 	    show_position(&this_label->place, 3);
-	    if (this_label->hypertext)
-		fprintf(stderr, " hypertext");
-	    switch (this_label->pos) {
-	    case LEFT:{
-		    fputs(" left", stderr);
-		    break;
-		}
-	    case CENTRE:{
-		    fputs(" centre", stderr);
-		    break;
-		}
-	    case RIGHT:{
-		    fputs(" right", stderr);
-		    break;
-		}
-	    }
-	    if (this_label->rotate)
-	    	fprintf(stderr, " rotated by %d degrees (if possible)", this_label->rotate);
-	    else
-	    	fprintf(stderr, " not rotated");
-	    fprintf(stderr, " %s ", this_label->layer ? "front" : "back");
-	    if (this_label->font != NULL)
-		fprintf(stderr, " font \"%s\"", this_label->font);
-	    if (this_label->textcolor.type)
-		save_textcolor(stderr, &this_label->textcolor);
-	    if (this_label->noenhanced)
-		fprintf(stderr, " noenhanced");
-	    if ((this_label->lp_properties.flags & LP_SHOW_POINTS) == 0)
-		fprintf(stderr, " nopoint");
-	    else {
-		fprintf(stderr, " point with color of");
-		save_linetype(stderr, &(this_label->lp_properties), TRUE);
-		fprintf(stderr, " offset ");
-		show_position(&this_label->offset, 3);
-	    }
-
-	    if (this_label->boxed) {
-		fprintf(stderr," boxed");
-		if (this_label->boxed > 0)
-		    fprintf(stderr," bs %d",this_label->boxed);
-	    }
-
-	    /* Entry font added by DJL */
+	    save_label_style(stderr, this_label);
 	    fputc('\n', stderr);
 	}
     }
@@ -1850,20 +1902,6 @@ show_arrow(int tag)
 	int_error(c_token, "arrow not found");
 }
 
-
-/* process 'show keytitle' command */
-static void
-show_keytitle()
-{
-    legend_key *key = &keyT;
-    SHOW_ALL_NL;
-
-    fprintf(stderr, "\tkey title is \"%s\"\n", conv_text(key->title.text));
-    if (key->title.font && *(key->title.font))
-	fprintf(stderr,"\t  font \"%s\"\n", key->title.font);
-}
-
-
 /* process 'show key' command */
 static void
 show_key()
@@ -1925,11 +1963,16 @@ show_key()
 		break;
 	    }
 	}
+	fputs("  offset: ", stderr);
+	show_position(&key->offset, 2);
 	fputs("\n", stderr);
 	break;
     }
     case GPKEY_USER_PLACEMENT:
-	fputs("\tkey is at ", stderr);
+	fprintf(stderr, "\t%s %s of ",
+		key->vpos == JUST_BOT ? "bottom" : key->vpos == JUST_CENTRE ? "center" : "top",
+		key->hpos == RIGHT ? "right" : key->hpos == LEFT ? "left" : "center");
+	fputs("key is at ", stderr);
 	show_position(&key->user_pos, 2);
 	putc('\n', stderr);
 	break;
@@ -1955,6 +1998,13 @@ show_key()
 	fprintf(stderr," \n");
     }
 
+    if (key->user_width.x > 0) {
+	fprintf(stderr, "\
+\tThe total key width is required to be ");
+	show_position(&(key->user_width), 1);
+	fprintf(stderr, "\n");
+    }
+
     fprintf(stderr, "\
 \tsample length is %g characters\n\
 \tvertical spacing is %g characters\n\
@@ -1970,11 +2020,15 @@ show_key()
 	    key->auto_titles == COLUMNHEAD_KEYTITLES
 	    ? "with column header" : "");
 
-    fputs("\tmaximum number of columns is ", stderr);
-    if (key->maxcols > 0)
-	fprintf(stderr, "%d for horizontal alignment\n", key->maxcols);
-    else
-	fputs("calculated automatically\n", stderr);
+    if (key->user_cols > 0) {
+	fprintf(stderr, "\trequired number of columns: %d\n", key->user_cols);
+    } else {
+	fputs("\tmaximum number of columns is ", stderr);
+	if (key->maxcols > 0)
+	    fprintf(stderr, "%d for horizontal alignment\n", key->maxcols);
+	else
+	    fputs("calculated automatically\n", stderr);
+    }
     fputs("\tmaximum number of rows is ", stderr);
     if (key->maxrows > 0)
 	fprintf(stderr, "%d for vertical alignment\n", key->maxrows);
@@ -1983,12 +2037,17 @@ show_key()
     if (key->font && *(key->font))
 	fprintf(stderr,"\t  font \"%s\"\n", key->font);
     if (key->textcolor.type != TC_LT || key->textcolor.lt != LT_BLACK) {
-	fputs("\t ", stderr);
+	fputs("\tentry", stderr);
 	save_textcolor(stderr, &(key->textcolor));
 	fputs("\n", stderr);
     }
 
-    show_keytitle();
+    if (key->title.text == NULL)
+	fprintf(stderr, "\n\tno key title\n");
+    else {
+	fprintf(stderr, "\n\tkey title is ");
+	save_keytitle(stderr);
+    }
 }
 
 
@@ -2167,114 +2226,51 @@ show_palette_rgbformulae()
     ++c_token;
 }
 
-
-static void
-show_palette_fit2rgbformulae()
-{
-#define rgb_distance(r,g,b) ((r)*(r) + (g)*(g) + (b)*(b))
-    int pts = 32; /* resolution: nb of points in the discrete raster for comparisons */
-    int i, p, ir, ig, ib;
-    int rMin=0, gMin=0, bMin=0;
-    int maxFormula = sm_palette.colorFormulae - 1; /* max formula number */
-    double gray, dist, distMin;
-    rgb_color *currRGB;
-    int *formulaeSeq;
-    double **formulae;
-    ++c_token;
-    if (sm_palette.colorMode == SMPAL_COLOR_MODE_RGB && sm_palette.cmodel == C_MODEL_RGB) {
-	fprintf(stderr, "\tCurrent palette is\n\t    set palette rgbformulae %i,%i,%i\n", sm_palette.formulaR, sm_palette.formulaG, sm_palette.formulaB);
-	return;
-    }
-    /* allocate and fill R, G, B values rastered on pts points */
-    currRGB = (rgb_color*)gp_alloc(pts * sizeof(rgb_color), "RGB pts");
-    for (p = 0; p < pts; p++) {
-	gray = (double)p / (pts - 1);
-	rgb1_from_gray(gray, &(currRGB[p]));
-    }
-    /* organize sequence of rgb formulae */
-    formulaeSeq = gp_alloc((2*maxFormula+1) * sizeof(int), "formulaeSeq");
-    for (i = 0; i <= maxFormula; i++)
-	formulaeSeq[i] = i;
-    for (i = 1; i <= maxFormula; i++)
-	formulaeSeq[maxFormula+i] = -i;
-    /* allocate and fill all +-formulae on the interval of given number of points */
-    formulae = gp_alloc((2*maxFormula+1) * sizeof(double*), "formulae");
-    for (i = 0; i < 2*maxFormula+1; i++) {
-	formulae[i] = gp_alloc(pts * sizeof(double), "formulae pts");
-	for (p = 0; p < pts; p++) {
-	    double gray = (double)p / (pts - 1);
-	    formulae[i][p] = GetColorValueFromFormula(formulaeSeq[i], gray);
-	}
-    }
-    /* Now go over all rastered formulae, compare them to the current one, and
-       find the minimal distance.
-     */
-    distMin = VERYLARGE;
-    for (ir = 0; ir <	 2*maxFormula+1; ir++) {
-	for (ig = 0; ig < 2*maxFormula+1; ig++) {
-	    for (ib = 0; ib < 2*maxFormula+1; ib++) {
-		dist = 0; /* calculate distance of the two rgb profiles */
-		for (p = 0; p < pts; p++) {
-		double tmp = rgb_distance(
-			    currRGB[p].r - formulae[ir][p],
-			    currRGB[p].g - formulae[ig][p],
-			    currRGB[p].b - formulae[ib][p] );
-		    dist += tmp;
-		}
-		if (dist < distMin) {
-		    distMin = dist;
-		    rMin = formulaeSeq[ir];
-		    gMin = formulaeSeq[ig];
-		    bMin = formulaeSeq[ib];
-		}
-	    }
-	}
-    }
-    fprintf(stderr, "\tThe best match of the current palette corresponds to\n\t    set palette rgbformulae %i,%i,%i\n", rMin, gMin, bMin);
-#undef rgb_distance
-    for (i = 0; i < 2*maxFormula+1; i++)
-	free(formulae[i]);
-    free(formulae);
-    free(formulaeSeq);
-    free(currRGB);
-}
-
-
 static void
 show_palette_palette()
 {
-    int colors, i;
+    int i;
+    int colors = 128;
     double gray;
     rgb_color rgb1;
     rgb255_color rgb255;
-    int how = 0; /* How to print table: 0: default large; 1: rgb 0..1; 2: integers 0..255 */
     FILE *f;
+    /* How to format the table:
+	0: 1. gray=0.1111, (r,g,b)=(0.3333,0.0014,0.6428), #5500a4 =  85   0 164
+	1: 0.3333  0.0014  0.6428
+	2: 85      0       164
+	3: 0x5500a4
+      */
+    int format = 0;
 
     c_token++;
-    if (END_OF_COMMAND)
-	int_error(c_token,"palette size required");
-    colors = int_expression();
-    if (colors<2) colors = 128;
-    if (!END_OF_COMMAND) {
-	if (almost_equals(c_token, "f$loat")) /* option: print r,g,b floats 0..1 values */
-	    how = 1;
-	else if (almost_equals(c_token, "i$nt")) /* option: print only integer 0..255 values */
-	    how = 2;
-    else
-	    int_error(c_token, "expecting no option or int or float");
-	c_token++;
+    while (!END_OF_COMMAND) {
+	if (equals(c_token, "float")) {
+	    format = 1;
+	    c_token++;
+	} else if (equals(c_token, "int")) {
+	    format = 2;
+	    c_token++;
+	} else if (equals(c_token, "hex")) {
+	    format = 3;
+	    c_token++;
+	} else {
+	    colors = int_expression();
+	    if (colors < 2)
+		colors = 128;
+	}
     }
 
-    i = (print_out==NULL || print_out==stderr || print_out==stdout);
     f = (print_out) ? print_out : stderr;
     fprintf(stderr, "%s palette with %i discrete colors",
-	    (sm_palette.colorMode == SMPAL_COLOR_MODE_GRAY) ? "Gray" : "Color", colors);
-    if (!i)
+	(sm_palette.colorMode == SMPAL_COLOR_MODE_GRAY) ? "Gray" : "Color", colors);
+    if (print_out_name)
 	fprintf(stderr," saved to \"%s\".", print_out_name);
-    else
-	fprintf(stderr, ".\n");
+    fprintf(stderr,"\n");
 
     for (i = 0; i < colors; i++) {
+	char line[80];
+
 	/* colours equidistantly from [0,1]  */
 	gray = (double)i / (colors - 1);
 	if (sm_palette.positive == SMPAL_NEGATIVE)
@@ -2282,20 +2278,28 @@ show_palette_palette()
 	rgb1_from_gray(gray, &rgb1);
 	rgb255_from_rgb1(rgb1, &rgb255);
 
-	switch (how) {
+	switch (format) {
 	    case 1:
-		fprintf(f, "%0.4f\t%0.4f\t%0.4f\n", rgb1.r, rgb1.g, rgb1.b);
+		sprintf(line, "%0.4f\t%0.4f\t%0.4f", rgb1.r, rgb1.g, rgb1.b);
 		break;
 	    case 2:
-		fprintf(f, "%i\t%i\t%i\n", (int)rgb255.r, (int)rgb255.g, (int)rgb255.b);
+		sprintf(line, "%i\t%i\t%i", (int)rgb255.r, (int)rgb255.g, (int)rgb255.b);
+		break;
+	    case 3:
+		sprintf(line, "0x%06x", (int)rgb255.r<<16 | (int)rgb255.g<<8 | (int)rgb255.b);
 		break;
 	    default:
-		fprintf(f,
-    		    "%3i. gray=%0.4f, (r,g,b)=(%0.4f,%0.4f,%0.4f), #%02x%02x%02x = %3i %3i %3i\n",
+		sprintf(line,
+		    "%3i. gray=%0.4f, (r,g,b)=(%0.4f,%0.4f,%0.4f), #%02x%02x%02x = %3i %3i %3i",
     		    i, gray, rgb1.r, rgb1.g, rgb1.b,
     		    (int)rgb255.r, (int)rgb255.g, (int)rgb255.b,
     		    (int)rgb255.r, (int)rgb255.g, (int)rgb255.b );
+		break;
 	}
+	if (print_out_var)
+	    append_to_datablock( &print_out_var->udv_value, strdup(line) );
+	else
+	    fprintf(f, "%s\n", line);
     }
 }
 
@@ -2329,7 +2333,6 @@ show_palette_gradient()
 static void
 show_colornames(const struct gen_table *tbl)
 {
-    int i=0;
     while (tbl->key) {
 	/* Print color names and their rgb values, table with 1 column */
 	int r = ((tbl->value >> 16 ) & 255);
@@ -2339,7 +2342,6 @@ show_colornames(const struct gen_table *tbl)
 	fprintf( stderr, "\n  %-18s ", tbl->key );
 	fprintf(stderr, "#%02x%02x%02x = %3i %3i %3i", r,g,b, r,g,b);
 	++tbl;
-	++i;
     }
     fputs( "\n", stderr );
     ++c_token;
@@ -2372,6 +2374,9 @@ show_palette()
 	  case SMPAL_COLOR_MODE_GRADIENT:
 	    fputs( "\tcolor mapping by defined gradient\n", stderr );
 	    break;
+	  case SMPAL_COLOR_MODE_VIRIDIS:
+	    fputs( "\tgradient named viridis\n", stderr );
+	    break;
 	  case SMPAL_COLOR_MODE_FUNCTIONS:
 	    fputs("\tcolor mapping is done by user defined functions\n",stderr);
 	    if (sm_palette.Afunc.at && sm_palette.Afunc.definition)
@@ -2401,12 +2406,23 @@ show_palette()
 	else
 	    fputs("ALL remaining", stderr);
 	fputs(" color positions for discrete palette terminals\n", stderr);
+	if (sm_palette.colors > 0)
+	    fprintf(stderr,"\t\t(current terminal has provided %d)\n", sm_palette.colors);
 	fputs( "\tColor-Model: ", stderr );
 	switch( sm_palette.cmodel ) {
-	default:
-	case C_MODEL_RGB: fputs( "RGB\n", stderr ); break;
-	case C_MODEL_HSV: fputs( "HSV\n", stderr ); break;
-	case C_MODEL_CMY: fputs( "CMY\n", stderr ); break;
+	    default:
+	    case C_MODEL_RGB:
+		fputs( "RGB\n", stderr );
+		break;
+	    case C_MODEL_CMY:
+		fputs( "CMY\n", stderr );
+		break;
+	    case C_MODEL_HSV:
+		if (sm_palette.HSV_offset != 0)
+		    fprintf(stderr,"HSV start %.2f\n",sm_palette.HSV_offset);
+		else
+		    fputs( "HSV\n", stderr );
+		break;
 	}
 	fprintf(stderr,"\tgamma is %.4g\n", sm_palette.gamma );
 	return;
@@ -2432,11 +2448,6 @@ show_palette()
         show_palette_colornames();
 	return;
     }
-    else if (almost_equals(c_token, "fit2rgb$formulae" )) {
-        /* 'show palette fit2rgbformulae' */
-	show_palette_fit2rgbformulae();
-	return;
-    }
     else { /* wrong option to "show palette" */
         int_error( c_token, "Expecting 'gradient' or 'palette <n>' or 'rgbformulae' or 'colornames'");
     }
@@ -2533,12 +2544,6 @@ show_pm3d()
 	save_linetype(stderr, &(pm3d.border), FALSE);
 	fprintf(stderr,"\n");
     }
-    if (pm3d_shade.strength > 0) {
-	fprintf(stderr,"\tlighting primary component %g specular component %g",
-		pm3d_shade.strength, pm3d_shade.spec);
-	fprintf(stderr," second spot contribution %g\n",
-		pm3d_shade.spec2);
-    }
     fprintf(stderr,"\tsteps for bilinear interpolation: %d,%d\n",
 	 pm3d.interp_i, pm3d.interp_j);
     fprintf(stderr,"\tquadrangle color according to ");
@@ -2552,6 +2557,10 @@ show_pm3d()
 	case PM3D_WHICHCORNER_RMS: fputs("root mean square of 4 corners\n", stderr); break;
 	default: fprintf(stderr, "corner %i\n", pm3d.which_corner_color - PM3D_WHICHCORNER_C1 + 1);
     }
+    if (pm3d_shade.strength > 0) {
+	fprintf(stderr, "\tLighting model:\n");
+	save_pm3d_lighting(stderr, "\t");
+    }
 }
 
 
@@ -2610,6 +2619,14 @@ show_decimalsign()
     fprintf(stderr, "\tdegree sign for output is %s \n", degree_sign);
 }
 
+static void
+show_imaginary_i()
+{
+    SHOW_ALL_NL;
+    fprintf(stderr, "\timaginary component of gprintf format %%Ci indicated by %s\n",
+	imaginary_user ? imaginary_user : "i or for latex terminals \\imath");
+}
+
 /* process 'show micro' command */
 static void
 show_micro()
@@ -2617,6 +2634,7 @@ show_micro()
     SHOW_ALL_NL;
 
     fprintf(stderr, "\tmicro character for output is %s \n",
+	(use_micro && micro_user) ? micro_user :
     	(use_micro && micro) ? micro : "u");
 }
 
@@ -2732,7 +2750,27 @@ static void
 show_polar()
 {
     SHOW_ALL_NL;
-    fprintf(stderr, "\tpolar is %s\n", (polar) ? "ON" : "OFF");
+    fprintf(stderr, "\tpolar mode is %s\n", (polar) ? "ON" : "OFF");
+#ifdef USE_POLAR_GRID
+    if (1) {
+	fprintf(stderr,"\tpolar grid uses %d theta wedges and %d radial segments\n",
+		polar_grid.theta_segments, polar_grid.r_segments);
+	fprintf(stderr,"\tmasked by theta range [%g:%g] radial range [%g:",
+		THETA_AXIS.min, THETA_AXIS.max, polar_grid.rmin);
+	if (polar_grid.rmax < VERYLARGE)
+		fprintf(stderr,"%g]\n",polar_grid.rmax);
+	else
+		fprintf(stderr,"*]\n");
+	fprintf(stderr,"\tpolar gridding scheme %s ",
+		reverse_table_lookup(dgrid3d_mode_tbl, polar_grid.mode));
+	if (polar_grid.mode == DGRID3D_QNORM)
+		fprintf(stderr,"%d\n", polar_grid.norm_q);
+	else
+		fprintf(stderr,"%s scale %g\n",
+			polar_grid.kdensity ? "kdensity" : "", polar_grid.scale);
+    } else
+	fprintf(stderr,"\tno polar gridding\n");
+#endif /* USE_POLAR_GRID */
 }
 
 
@@ -2798,8 +2836,11 @@ static void
 show_surface()
 {
     SHOW_ALL_NL;
-    fprintf(stderr, "\tsurface is %sdrawn %s\n",
-	draw_surface ? "" : "not ", implicit_surface ? "" : "only if explicitly requested");
+    if (!draw_surface)
+	fprintf(stderr, "\tsurface is not drawn\n");
+    else
+	fprintf(stderr, "\tsurface is drawn %s\n",
+	    implicit_surface ? "" : "only if explicitly requested");
 }
 
 
@@ -2817,11 +2858,15 @@ show_hidden3d()
 static void
 show_increment()
 {
+#ifdef BACKWARD_COMPATIBILITY
     fprintf(stderr,"\tPlot lines increment over ");
     if (prefer_line_styles)
 	fprintf(stderr, "user-defined line styles rather than default line types\n");
     else
 	fprintf(stderr, "default linetypes\n");
+#else
+    fprintf(stderr,"\t'set style increment' is deprecated\n");
+#endif
 }
 
 static void
@@ -2858,14 +2903,26 @@ show_size()
 
     fprintf(stderr, "\tsize is scaled by %g,%g\n", xsize, ysize);
     if (aspect_ratio > 0)
-	fprintf(stderr, "\tTry to set aspect ratio to %g:1.0\n", aspect_ratio);
+	fprintf(stderr, "\tTry to set aspect ratio to %g:1\n", aspect_ratio);
     else if (aspect_ratio == 0)
 	fputs("\tNo attempt to control aspect ratio\n", stderr);
     else
-	fprintf(stderr, "\tTry to set LOCKED aspect ratio to %g:1.0\n",
-		-aspect_ratio);
+	fprintf(stderr, "\tTry to set ratio of x and y axis scales to %g:1 %s\n",
+		-aspect_ratio,
+		aspect_ratio == -1 ? "[isotropic axis scales]" : "");
 }
 
+static void
+show_isotropic()
+{
+    SHOW_ALL_NL;
+    if (aspect_ratio == -1)
+	fprintf(stderr, "\tin 2D x and y axis scales are the same\n");
+    if (aspect_ratio_3D == 2)
+	fprintf(stderr, "\tin 3D x and y axis scales are the same\n");
+    if (aspect_ratio_3D == 3)
+	fprintf(stderr, "\tin 3D x, y, and z axis scales are the same\n");
+}
 
 /* process 'show origin' command */
 static void
@@ -2936,17 +2993,26 @@ show_mtics(struct axis *axis)
     case MINI_DEFAULT:
 	fprintf(stderr, "\
 \tminor %stics are off for linear scales\n\
+\tminor %stics are off for time axes\n\
 \tminor %stics are computed automatically for log scales\n",
-	name, name);
+	name, name, name);
 	break;
     case MINI_AUTO:
 	fprintf(stderr, "\tminor %stics are computed automatically\n", name);
 	break;
     case MINI_USER:
 	fprintf(stderr, "\
-\tminor %stics are drawn with %d subintervals between major xtic marks\n",
-		name, (int) axis->mtic_freq);
+\tminor %stics are drawn with %d subintervals between major tic marks\n",
+		name, axis->mtic_freq);
 	break;
+    case MINI_TIME:
+	{
+	char *timelevel = clean_reverse_table_lookup(timelevels_tbl, axis->minitic_units);
+	fprintf(stderr, "\tminor %stics drawn every %d %s\n",
+		name, axis->mtic_freq, timelevel);
+	free(timelevel);
+	break;
+	}
     default:
 	int_error(NO_CARET, "Unknown minitic type in show_mtics()");
     }
@@ -2975,6 +3041,21 @@ show_range(AXIS_INDEX axis)
     save_prange(stderr, axis_array + axis);
 }
 
+static void
+show_text_justification( JUSTIFY just )
+{
+    switch (just) {
+    case LEFT:
+	fprintf(stderr, "left justified ");
+	break;
+    case RIGHT:
+	fprintf(stderr, "right justified ");
+	break;
+    case CENTRE:
+	fprintf(stderr, "center justified ");
+	break;
+    }
+}
 
 /* called by the functions below */
 static void
@@ -2984,21 +3065,19 @@ show_xyzlabel(const char *name, const char *suffix, text_label *label)
 	fprintf(stderr, "\t%s%s is \"%s\", offset at ", name, suffix,
 	    label->text ? conv_text(label->text) : "");
 	show_position(&label->offset, 3);
-	fprintf(stderr, label->pos == LEFT ? " left justified"
-			: label->pos == RIGHT ? " right justified"
-			: "");
+	show_text_justification(label->pos);
     } else
 	return;
 
     if (label->font)
 	fprintf(stderr, ", using font \"%s\"", conv_text(label->font));
 
-    if (label->tag == ROTATE_IN_3D_LABEL_TAG)
+    if (label->tag == LABEL_TAG_ROTATE_IN_3D)
 	fprintf(stderr, ", parallel to axis in 3D plots");
     else if (label->rotate)
-	fprintf(stderr, ", rotated by %d degrees in 2D plots", label->rotate);
+	fprintf(stderr, ", rotated by %g degrees in 2D plots", label->rotate);
 
-    if (label->textcolor.type)
+    if (label->textcolor.type != TC_DEFAULT)
 	save_textcolor(stderr, &label->textcolor);
 
     if (label->noenhanced)
@@ -3243,6 +3322,8 @@ show_plot()
 {
     SHOW_ALL_NL;
     fprintf(stderr, "\tlast plot command was: %s\n", replot_line);
+    if (last_plot_was_multiplot)
+	fprintf(stderr, "\tlast plot was a multiplot (see $GPVAL_LAST_MULTIPLOT)\n");
 }
 
 
@@ -3290,7 +3371,9 @@ show_variables()
 	    fprintf(stderr, "\t%-*s ", len, udv->udv_name);
 	    fputs("= ", stderr);
 	    disp_value(stderr, &(udv->udv_value), TRUE);
-	    (void) putc('\n', stderr);
+	    if (udv->locality > 0)
+		fprintf(stderr, "\t(locality %d)", udv->locality);
+	    putc('\n', stderr);
 	}
 	udv = udv->next_udv;
     }
@@ -3319,17 +3402,26 @@ show_linestyle(int tag)
 
 /* Show linetype number <tag> (0 means show all) */
 static void
-show_linetype(struct linestyle_def *listhead, int tag)
+show_linetype(struct linestyle_def *listhead)
 {
     struct linestyle_def *this_linestyle;
     TBOOLEAN showed = FALSE;
     int recycle_count = 0;
+    int tag = 0;	/* default to show all defined linetypes */
+
+    if (!END_OF_COMMAND)
+	tag = int_expression();
+    if (tag <= 0) {
+	tag = 0;
+	fprintf(stderr, "\n\tlinetype  0   default zeroaxis (terminal-specific, dotted if possible)");
+    }
+    (void) putc('\n',stderr);
 
     for (this_linestyle = listhead; this_linestyle != NULL;
 	 this_linestyle = this_linestyle->next) {
 	if (tag == 0 || tag == this_linestyle->tag) {
 	    showed = TRUE;
-	    fprintf(stderr, "\tlinetype %d, ", this_linestyle->tag);
+	    fprintf(stderr, "\tlinetype %2d, ", this_linestyle->tag);
 	    save_linetype(stderr, &(this_linestyle->lp_properties), TRUE);
 	    fputc('\n', stderr);
 	}
@@ -3342,9 +3434,12 @@ show_linetype(struct linestyle_def *listhead, int tag)
     else if (listhead == first_mono_linestyle)
 	recycle_count = mono_recycle_count;
 
-    if (tag == 0 && recycle_count > 0)
-	fprintf(stderr, "\tLinetypes repeat every %d unless explicitly defined\n",
-		recycle_count);
+    if (tag == 0 && recycle_count > 0) {
+	fprintf(stderr,
+		"\tlinetypes > %d cyclically repeat the line properties, but not point properties,\n"
+		"\t              of linetypes 1-%d unless explicitly set otherwise\n",
+		recycle_count, recycle_count);
+    }
 }
 
 
@@ -3434,22 +3529,9 @@ show_ticdefp(struct axis *this_axis)
     if (this_axis->ticdef.rangelimited && !spiderplot)
 	fprintf(stderr, "\n\t  tics are limited to data range");
     fputs("\n\t  labels are ", stderr);
-    if (this_axis->manual_justify) {
-    	switch (this_axis->tic_pos) {
-    	case LEFT:{
-		fputs("left justified, ", stderr);
-		break;
-	    }
-    	case RIGHT:{
-		fputs("right justified, ", stderr);
-		break;
-	    }
-    	case CENTRE:{
-		fputs("center justified, ", stderr);
-		break;
-	    }
-    	}
-    } else
+    if (this_axis->manual_justify)
+	show_text_justification(this_axis->tic_pos);
+    else
         fputs("justified automatically, ", stderr);
     fprintf(stderr, "format \"%s\"", ticfmt);
     fprintf(stderr, "%s",
@@ -3460,7 +3542,7 @@ show_ticdefp(struct axis *this_axis)
 	fprintf(stderr,"  noenhanced");
     if (this_axis->tic_rotate) {
 	fprintf(stderr," rotated");
-	fprintf(stderr," by %d",this_axis->tic_rotate);
+	fprintf(stderr," by %g",this_axis->tic_rotate);
 	fputs(" in 2D mode, terminal permitting,\n\t", stderr);
     } else
 	fputs(" and are not rotated,\n\t", stderr);
@@ -3487,8 +3569,10 @@ show_ticdefp(struct axis *this_axis)
 		fputs(" from ", stderr);
 		save_num_or_time_input(stderr, this_axis->ticdef.def.series.start, this_axis);
 	    }
-	    fprintf(stderr, " by %g%s", this_axis->ticdef.def.series.incr,
-		    this_axis->datatype == DT_TIMEDATE ? " secs" : "");
+	    fprintf(stderr, " by %g %s", this_axis->ticdef.def.series.incr,
+		(this_axis->tictype == DT_TIMEDATE)
+		    ? clean_reverse_table_lookup(timelevels_tbl, this_axis->tic_units)
+		    : "");
 	    if (this_axis->ticdef.def.series.end != VERYLARGE) {
 		fputs(" until ", stderr);
 		save_num_or_time_input(stderr, this_axis->ticdef.def.series.end, this_axis);
@@ -3531,6 +3615,15 @@ show_ticdefp(struct axis *this_axis)
     }
 }
 
+static void
+show_warnings()
+{
+    if (suppress_warnings)
+	fprintf(stderr, "\tno warnings are printed\n");
+    else
+	fprintf(stderr, "\twarnings are printed to stderr\n");
+}
+
 /* called by show_tics */
 static void
 show_ticdef(AXIS_INDEX axis)
