@@ -137,8 +137,8 @@ void *term_interlock = NULL;
 /* true if "set monochrome" */
 TBOOLEAN monochrome = FALSE;
 
-/* true if in multiplot mode */
-TBOOLEAN multiplot = FALSE;
+/* 1 + stack depth at which "set multiplot" occurred" */
+int in_multiplot = 0;
 int multiplot_count = 0;
 
 /* text output encoding, for terminals that support it */
@@ -196,6 +196,9 @@ int mono_recycle_count = 0;
 
 /* Internal variables */
 
+/* cleared on entry when terminal is first initialised */
+TBOOLEAN term_on_entry = TRUE;
+
 /* true if terminal is in graphics mode */
 static TBOOLEAN term_graphics = FALSE;
 
@@ -215,15 +218,15 @@ static double term_pointsize=1;
 
 static void term_suspend(void);
 static void term_close_output(void);
+static void term_get_termsize(int *xsize, int *ysize);
 
 static void null_linewidth(double);
 static void do_point(unsigned int x, unsigned int y, int number);
 static void do_pointsize(double size);
-static void line_and_point(unsigned int x, unsigned int y, int number);
 static void do_arrow(unsigned int sx, unsigned int sy, unsigned int ex, unsigned int ey, int headstyle);
 static void null_dashtype(int type, t_dashtype *custom_dash_pattern);
 
-static int null_text_angle(int ang);
+static int null_text_angle(float ang);
 static int null_justify_text(enum JUSTIFY just);
 static int null_scale(double x, double y);
 static void null_layer(t_termlayer layer);
@@ -339,7 +342,7 @@ term_set_output(char *dest)
     FPRINTF((stderr, "term_set_output\n"));
     assert(dest == NULL || dest != outstr);
 
-    if (multiplot) {
+    if (in_multiplot > 0) {
 	fputs("In multiplot mode you can't change the output\n", stderr);
 	return;
     }
@@ -496,30 +499,54 @@ term_initialise()
     }
 }
 
+/*
+ * The kitty terminals (kittycairo and kittygd) would like to default to
+ * using the full width of the terminal.  Use an ioctl to query fd 0
+ * on the assumption that it is STDIN and it is connected to the user's
+ * terminal window.
+ */
+static void
+term_get_termsize(int *xsize, int *ysize)
+{
+#if defined(HAVE_SYS_IOCTL_H) && defined(TIOCGWINSZ)
+# include <sys/ioctl.h>
+    struct winsize sz;
+    int ierr = ioctl(0, TIOCGWINSZ, &sz);
+    if (ierr >= 0 && sz.ws_xpixel > 0 && sz.ws_ypixel > 0) {
+	*xsize = sz.ws_xpixel;
+	*ysize = sz.ws_ypixel;
+    } else
+#endif
+    *xsize = *ysize = 0;
+}
+
 
 void
 term_start_plot()
 {
-    FPRINTF((stderr, "term_start_plot()\n"));
+    if (term_force_init) {
+	/* Used by kitty and webp terminals to support animation
+	 * (successive frames must start with a blank image).
+	 */
+	term->init();
+	term_force_init = FALSE;
+    } else
 
     if (!term_initialised)
 	term_initialise();
 
     if (!term_graphics) {
-	FPRINTF((stderr, "- calling term->graphics()\n"));
 	(*term->graphics) ();
 	term_graphics = TRUE;
-    } else if (multiplot && term_suspended) {
-	if (term->resume) {
-	    FPRINTF((stderr, "- calling term->resume()\n"));
+    } else if (in_multiplot && term_suspended) {
+	if (term->resume)
 	    (*term->resume) ();
-	}
 	term_suspended = FALSE;
     }
 
     sanity_check_font_size();
 
-    if (multiplot)
+    if (in_multiplot)
 	multiplot_count++;
 
     /* Sync point for epslatex text positioning */
@@ -531,10 +558,10 @@ term_start_plot()
 	invalidate_palette();
 
     /* Set canvas size to full range of current terminal coordinates */
-	canvas.xleft  = 0;
-	canvas.xright = term->xmax - 1;
-	canvas.ybot   = 0;
-	canvas.ytop   = term->ymax - 1;
+    canvas.xleft  = 0;
+    canvas.xright = term->xmax - 1;
+    canvas.ybot   = 0;
+    canvas.ytop   = term->ymax - 1;
 
 }
 
@@ -549,7 +576,7 @@ term_end_plot()
     /* Sync point for epslatex text positioning */
     (*term->layer)(TERM_LAYER_END_TEXT);
 
-    if (!multiplot) {
+    if (in_multiplot == 0) {
 	FPRINTF((stderr, "- calling term->text()\n"));
 	(*term->text) ();
 	term_graphics = FALSE;
@@ -668,8 +695,7 @@ term_apply_lp_properties(struct lp_style_type *lp)
     } else /* All normal lines will be solid unless a dashtype is given */
 	(*term->linetype) (LT_SOLID);
 
-    /* Version 5.4.2
-     * If the line is not wanted at all, setting dashtype can only hurt.
+    /* If the line is not wanted at all, setting dashtype can only hurt.
      * But we might still want to set a color so that points can use it.
      */
     if (lt == LT_NODRAW) {
@@ -686,10 +712,9 @@ term_apply_lp_properties(struct lp_style_type *lp)
     else if (dt == DASHTYPE_SOLID)
 	(*term->dashtype) (dt, NULL);
     else if (dt >= 0)
-	/* The null_dashtype() routine or a version 5 terminal's private  */
-	/* dashtype routine converts this into a call to term->linetype() */
-	/* yielding the same result as in version 4 except possibly for a */
-	/* different line width.					  */
+	/* The null_dashtype() routine or a terminal's private
+	 * dashtype routine converts this into a call to term->linetype()
+	 */
 	(*term->dashtype) (dt, NULL);
 
     /* Finally adjust the color of the line */
@@ -710,7 +735,7 @@ void
 term_end_multiplot()
 {
     FPRINTF((stderr, "term_end_multiplot()\n"));
-    if (!multiplot)
+    if (in_multiplot == 0)
 	return;
 
     if (term_suspended) {
@@ -771,7 +796,7 @@ write_multiline(
     char *text,
     JUSTIFY hor,                /* horizontal ... */
     VERT_JUSTIFY vert,          /* ... and vertical just - text in hor direction despite angle */
-    int angle,                  /* assume term has already been set for this */
+    float angle,               /* assume term has already been set for this */
     const char *font)           /* NULL or "" means use default */
 {
     struct termentry *t = term;
@@ -938,6 +963,7 @@ do_pointsize(double size)
 /*
  * general point routine
  */
+#if  defined(WITH_METAPOST) || defined(WITH_METAFONT) || defined(WITH_TEKTRONIX) || !defined(NO_BITMAP_SUPPORT)
 static void
 line_and_point(unsigned int x, unsigned int y, int number)
 {
@@ -947,6 +973,7 @@ line_and_point(unsigned int x, unsigned int y, int number)
     (*term->linetype) (NICE_LINE);
     do_point(x, y, number);
 }
+#endif
 
 /*
  * general arrow routine
@@ -1225,7 +1252,7 @@ do_arc(
    * 1 is vertical bottom to top (90 deg rotate)
  */
 static int
-null_text_angle(int ang)
+null_text_angle(float ang)
 {
     return (ang == 0);
 }
@@ -1268,7 +1295,7 @@ static void
 graphics_null()
 {
     fprintf(stderr,
-	    "WARNING: Plotting with an 'unknown' terminal.\n"
+	    "WARNING: Plotting with 'unknown' terminal.\n"
 	    "No output will be generated. Please select a terminal with 'set terminal'.\n");
 }
 
@@ -1334,6 +1361,89 @@ null_dashtype(int type, t_dashtype *custom_dash_pattern)
     term->linetype(type);
 }
 
+#ifdef USE_MOUSE
+
+/*
+ * This routine dummies up mouse events for terminals that can
+ * plot directly into the console (no separate graphics window).
+ * If "pause mouse" is active, it filters characters from stdin through
+ * the current mouse key bindings.  It attempts to handle arrow keys.
+ * Otherwise it returns the next character read from stdin.
+ *
+ * EAM - May 2023
+ *
+ * FIXME:  handle "pause -1" in additional to "pause mouse"
+ * FIXME:  can we somehow catch actual mouse events?
+ * FIXME:  how to detect shift/control/alt modifiers?
+ *	   empirical tests suggests <033><073><065> is control
+ *				    <033><073><062> is shift
+ */
+#if defined(HAVE_LIBREADLINE)
+  #define raw() rl_prep_terminal(1)
+  #define cook() rl_deprep_terminal()
+  #define nextchar() rl_getc(stdin)
+#elif defined(HAVE_LIBEDITLINE)
+  #define raw() rl_prep_terminal(1)
+  #define cook() rl_deprep_terminal()
+  #define nextchar() fgetc(stdin)
+#elif defined(READLINE)
+  #define raw() set_termio()
+  #define cook() reset_termio()
+  #define nextchar() fgetc(stdin)
+#endif
+
+int
+term_waitforinput(int options)
+{
+    int nextchar;
+
+    if (options == TERM_ONLY_CHECK_MOUSING)
+	return '\0';
+
+    if (paused_for_mouse) {
+	/* Read single character at a time */
+	raw();
+	nextchar = nextchar();
+
+	/* Normal exit from "pause mouse" */
+	if (nextchar == '\n' || nextchar == '\r') {
+	    paused_for_mouse = 0;
+	    cook();
+	    exec_event( GE_reset, 0, 0, 0, 0, 0 );
+	    return nextchar;
+	}
+
+	/* Try to catch arrow key escape sequences */
+	if (nextchar == '\033') {
+	    if ((nextchar = nextchar()) == '[') {
+		nextchar = nextchar();
+		switch (nextchar) {
+		case 'A':	nextchar = GP_Up; break;
+		case 'B':	nextchar = GP_Down; break;
+		case 'C':	nextchar = GP_Right; break;
+		case 'D':	nextchar = GP_Left; break;
+		default:	/* unrecognized */
+				return '\0';
+				break;
+		}
+	    } else {
+		/* Some unrecognized escape sequence */
+	    }
+	}
+
+	/* Feed to active key binding.
+	 * I.e. act as if this key had been pressed while focus was
+	 * in an active plot window (which it is, kind of).
+	 */
+	exec_event( GE_keypress, 0, 0, nextchar, 0, 0 );
+	return '\0';
+    } else {
+	read_and_return_character();
+    }
+}
+
+#endif
+
 /* setup the magic macros to compile in the right parts of the
  * terminal drivers included by term.h
  */
@@ -1447,7 +1557,8 @@ set_term()
 
     if (!t) {
 	change_term("unknown", 7);
-	int_error(c_token-1, "unknown or ambiguous terminal type; type just 'set terminal' for a list");
+	int_warn(c_token-1, "unknown or ambiguous terminal type; type 'set terminal' for a list");
+	return term;
     }
 
     /* otherwise the type was changed */
@@ -1479,6 +1590,11 @@ change_term(const char *origname, int length)
 	name = "epscairo";
 	length = 8;
     }
+    /* To allow "set term kitty" as short for "set term kittycairo" */
+    if (!strncmp(origname,"kitty",length)) {
+	name = "kittycairo";
+	length = 10;
+    }
 #endif
 #ifdef HAVE_LIBGD
     /* To allow "set term sixel" as short for "set term sixelgd" */
@@ -1510,6 +1626,8 @@ change_term(const char *origname, int length)
     term_initialised = FALSE;
 
     /* check that optional fields are initialised to something */
+    if (term->options == 0)
+	term->options = options_null;
     if (term->text_angle == 0)
 	term->text_angle = null_text_angle;
     if (term->justify_text == 0)
@@ -1535,7 +1653,7 @@ change_term(const char *origname, int length)
     if (term->dashtype == 0)
 	term->dashtype = null_dashtype;
 
-    if (interactive)
+    if (interactive && !term_on_entry)
 	fprintf(stderr, "\nTerminal type is now '%s'\n", term->name);
 
     /* Invalidate any terminal-specific structures that may be active */
@@ -1569,14 +1687,19 @@ init_terminal()
     gnuterm = getenv("GNUTERM");
     if (gnuterm != (char *) NULL) {
 	/* April 2017 - allow GNUTERM to include terminal options */
+	char *set_term_command;
+	char *semicolon = NULL;
 	char *set_term = "set term ";
-	char *set_term_command = gp_alloc(strlen(set_term) + strlen(gnuterm) + 4, NULL);
+	/* scrub any extraneous commands possibly hidden in GNUTERM */
+	if ( (semicolon = strchr(gnuterm, ';')) != NULL)
+	    *semicolon = '\0';
+	set_term_command = gp_alloc(strlen(set_term) + strlen(gnuterm) + 4, NULL);
 	strcpy(set_term_command, set_term);
 	strcat(set_term_command, gnuterm);
-	do_string(set_term_command);
-	free(set_term_command);
+	do_string_and_free(set_term_command);
 	/* replicate environmental variable GNUTERM for internal use */
 	Gstring(&(add_udv_by_name("GNUTERM")->udv_value), gp_strdup(gnuterm));
+	term_on_entry = FALSE;
 	return;
 
     } else {
@@ -1651,16 +1774,6 @@ init_terminal()
 	    term_name = "pm";
 #endif /*OS2 */
 
-/* set linux terminal only if LINUX_setup was successful, if we are on X11
-   LINUX_setup has failed, also if we are logged in by network */
-#ifdef LINUXVGA
-	if (LINUX_graphics_allowed)
-#if defined(VGAGL) && defined (THREEDKIT)
-	    term_name = "vgagl";
-#else
-	    term_name = "linux";
-#endif
-#endif /* LINUXVGA */
     }
 
     /* We have a name, try to set term type */
@@ -1682,16 +1795,20 @@ init_terminal()
 	if (change_term(term_name, namelength)) {
 	    if (strcmp(term->name,"x11"))
 		term->options();
+	    term_on_entry = FALSE;
 	    return;
 	}
 	fprintf(stderr, "Unknown or ambiguous terminal name '%s'\n", term_name);
     }
     change_term("unknown", 7);
+    term_on_entry = FALSE;
 }
 
 
-/* test terminal by drawing border and text */
-/* called from command test */
+/*
+ * test page for terminal capabilities and current properties
+ * (lines, points, fill, test, polygons, text rotation, ...).
+ */
 void
 test_term()
 {
@@ -1704,6 +1821,7 @@ test_term()
     int p_width;
     TBOOLEAN already_in_enhanced_text_mode;
     static t_colorspec black = BLACK_COLORSPEC;
+    struct lp_style_type ls = DEFAULT_LP_STYLE_TYPE;
 
     already_in_enhanced_text_mode = t->flags & TERM_ENHANCED_TEXT;
     if (!already_in_enhanced_text_mode)
@@ -1747,6 +1865,7 @@ test_term()
 	(*t->put_text) (x0 + t->h_char * 2, y0 + ymax_t - t->v_char * 2.25, tbuf);
     }
 
+    /* axis line type (lt 0) used for xzeroaxis, yzeroaxis, should always be dotted */
     (*t->linetype) (LT_AXIS);
     (*t->move) (x0 + xmax_t / 2, y0);
     (*t->vector) (x0 + xmax_t / 2, y0 + ymax_t - 1);
@@ -1799,6 +1918,15 @@ test_term()
 	(*t->set_font)("");
 	if (!already_in_enhanced_text_mode)
 	    do_string("set termopt noenh");
+    } else if (t->flags & TERM_IS_LATEX) {
+	char *tmptext1 =   "\\LaTeX:  $\\int{x_{0}^{n+1}}$";
+	(*t->put_text) (x0 + xmax_t * 0.5, y0 + ymax_t * 0.37, tmptext1);
+    }
+
+    /* Test for UTF8 support */
+    {
+	char *tmptext1 = "utf8: Å→∞";
+	(*t->put_text) (x0 + xmax_t * 0.5, y0 + ymax_t * 0.30, tmptext1);
     }
 
     /* test justification */
@@ -1820,7 +1948,7 @@ test_term()
 			y0 + ymax_t / 2 + t->v_char * 4, str);
 
     /* test tic size */
-    (*t->linetype)(2);
+    (*t->linetype)(LT_BLACK);
     (*t->move) ((unsigned int) (x0 + xmax_t / 2 + t->h_tic * (1 + axis_array[FIRST_X_AXIS].ticscale)), y0 + (unsigned int) ymax_t - 1);
     (*t->vector) ((unsigned int) (x0 + xmax_t / 2 + t->h_tic * (1 + axis_array[FIRST_X_AXIS].ticscale)),
 		  (unsigned int) (y0 + ymax_t - axis_array[FIRST_X_AXIS].ticscale * t->v_tic));
@@ -1832,15 +1960,14 @@ test_term()
 		    y0 + (unsigned int) (ymax_t - t->v_char),
 		    "show ticscale");
     (void) (*t->justify_text) (LEFT);
-    (*t->linetype)(LT_BLACK);
 
-    /* test line and point types */
+    /*
+     * line and point types
+     */
     x = x0 + xmax_t - t->h_char * 7 - p_width;
     y = y0 + ymax_t - key_entry_height;
     (*t->pointsize) (pointsize);
     for (i = -2; y > y0 + key_entry_height; i++) {
-	struct lp_style_type ls = DEFAULT_LP_STYLE_TYPE;
-	ls.l_width = 1;
 	load_linetype(&ls,i+1);
 	term_apply_lp_properties(&ls);
 
@@ -1857,8 +1984,9 @@ test_term()
     }
 
     /* test arrows (should line up with rotated text) */
-    (*t->linewidth) (1.0);
-    (*t->linetype) (0);
+    load_linetype(&ls, 2);
+    ls.l_width = 1;
+    term_apply_lp_properties(&ls);
     (*t->dashtype) (DASHTYPE_SOLID, NULL);
     x = x0 + 2. * t->v_char;
     y = y0 + ymax_t/2;
@@ -1874,7 +2002,6 @@ test_term()
     curr_arrow_headfilled = i;
 
     /* test text angle (should match arrows) */
-    (*t->linetype)(0);
     str = "rotated ce+ntred text";
     if ((*t->text_angle) (TEXT_VERTICAL)) {
 	if ((*t->justify_text) (CENTRE))
@@ -1981,15 +2108,17 @@ test_term()
 		corners[n].x = corners[0].x;
 		corners[n].y = corners[0].y;
 		if (j == 0) {
-		    (*t->linetype)(2);
+		    load_linetype(&ls, 3);
+		    term_apply_lp_properties(&ls);
 		    corners->style = FS_OPAQUE;
 		} else {
-		    (*t->linetype)(1);
+		    load_linetype(&ls, 4);
+		    term_apply_lp_properties(&ls);
 		    corners->style = FS_TRANSPARENT_SOLID + (50<<4);
 		}
 		term->filled_polygon(n+1, corners);
 	    }
-	    str = "filled polygons:";
+	    str = "filled polygons";
 	} else
 	    str = "No filled polygons";
 	(*t->linetype)(LT_BLACK);
@@ -2367,9 +2496,9 @@ enhanced_recursion(
 		    break;
 		}
 
-	    /*     FIXME: non-utf8 environments not yet supported.
-	     *     Note that some terminals may have an alternative way to handle unicode
-	     *     escape sequences that is not dependent on encoding.
+	    /*     Non-utf8 environments not supported here.
+	     *     Note that some terminals have an alternative way to handle unicode
+	     *     escape sequences that is not dependent on gnuplot's encoding.
 	     *     E.g. svg and html output could convert to xml sequences &#xhhhh;
 	     *     For these cases we must retain the leading backslash so that the
 	     *     unicode escape sequence can be recognized by the terminal driver.
